﻿/**
 * @author Danny Davis | Visual Advance | danny@visualadvance.com
 *
 * I created this file mainly for myself as a reference for how to setup a simple papervision scene, and add different materials to an object. 
 * To see more of my work go to: http://blog.visualadvance.com
 * 
 * Please note that the class path for this file is set to "./classes". So in the fla if you go to "publish settings" then the "flash" tab then 
 * click "settings" you will see the custom path added
 *
 * This file is based off of Tyler Egeto's Papervision 2.0 Basic World Setup Tutorial. Many of the comments below are his since he did such a 
 * good job of explaining not only how, but why. See his work at www.TylerEgeto.com
 * 
 * If you need more details, the official documentation is here: http://papervision3d.googlecode.com/svn/trunk/as3/trunk/docs/index.html
 */

package 
{
	import flash.events.Event;
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import flash.display.Bitmap;
	import flash.display.BitmapData;

	//import papervision class files
	//Every papervision scene absolutly needs 4 things, a camera, Scene3D, BasicRenderEngine, Viewport3D
	import org.papervision3d.cameras.Camera3D;
	//import org.papervision3d.cameras.DebugCamera3D;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.view.Viewport3D;

	//load some optional classes that we will use in this example
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.materials.BitmapAssetMaterial;
	import org.papervision3d.materials.MovieAssetMaterial;
	import org.papervision3d.materials.BitmapMaterial;

	public class PapervisionSimple extends Sprite
	{
		//declare variables for your 4 manditory objects
		private var scene:Scene3D;//Scenes hold all of your 3D data and objects. Every 3D element we create will be added to our scene
		private var camera:Camera3D;//This is the "eyes" in your scene, very much like looking through a real video camera
		//private var camera:DebugCamera3D; // the debug camera is good for figuring out where you want to place your camera
		private var render:BasicRenderEngine;//This handels the rendering of your 3D scene
		private var viewport:Viewport3D;//This is where your scene is drawn to when rendered

		public var myObject:Cube;//This object is not mandatory, but we need to declare it outside of our function so we can talk to it later.

		//main constructor function
		public function PapervisionSimple ()
		{
			//build the viewport and add it to the stage
			//it accepts several parameters. The first two are width and height respectivly.
			//the third is a auto scale boolean, by setting it to true it will automatically set the size equal to the stage, so we will do that.
			//there are other parameters as well, but these are all we will use in this example.
			viewport=new Viewport3D(0,0,true);
			addChild (viewport);

			//build instances of the other 3 mandatories
			scene=new Scene3D();
			render=new BasicRenderEngine();
			camera=new Camera3D();
			//camera=new DebugCamera3D(viewport) ;

			//We can control a number of things about our camera, position, zoom, focus, target, rotation,
			//While flash typically works in two axis, (x and y) papervision adds a z axis to things. This describes how far away an object is. (depth)
			//New objects are added to the scene at position [0x,0y,0z]. So we need to move the camera back or we will never be able to see other objects
			camera.z=1000;
			camera.zoom=12;
			camera.focus=100;
			camera.rotationX=180;
			camera.rotationZ=180;

			//at this point our papervision world is set up. if we were to render things as is it would work perfectly,
			//however since we have added nothing to our scene, we wouldn't see anything.


			//Every object you create must have a texture applied to it. There are many types of textures depending on your need.
			// this material list is used to list hte materials to be applied to the cube we make. I have used a variety of different types of materials that I commonly use.

			var ml:MaterialsList=new MaterialsList();
			ml.addMaterial (makeColorMat(0xFF0000),'front');
			ml.addMaterial (makeBitmapMat(bitmapClip),'back');
			ml.addMaterial (makeMovieAssetMat("LibraryMovie"),'right');
			ml.addMaterial (makeBitmapAssetMat("picture1"),'left');
			ml.addMaterial (makeBitmapFileMat("assets/images/picture2.jpg"),'top');
			ml.addMaterial (makeColorMat(0x000000),'bottom');

			//Objects made up of triangles, and in those objects we can control the number of subdivisions (number of triangles)
			//The higher number of subdivisions increases the quality, but also slows down the render time.
			//Now we make the actual cube
			myObject=new Cube(ml,100,100,100,1,1,1);

			// convert the position of our object to something relative to the stage x, y (top left) rather than papervisions x,y (center of stage)
			//convertPosition (myObject,{x:0,y:0},{width:100,height:100});

			//we then need to add the object to our scene or the camera will not see it
			scene.addChild (myObject);

			//The render engine works on an enterframe loop, so create one now
			addEventListener (Event.ENTER_FRAME,onEnter);
		}
		//----------------------------------------------------------------------------------------------------
		// custom functions for easily making the materials
		// these functions are not a part of papervision by default. I jsut wrote them to make creating materials easier on myself.
		
		// color
		private function makeColorMat (myColor:Number):ColorMaterial
		{
			var tempMat:ColorMaterial = new ColorMaterial(myColor);
			return tempMat;
		}
		
		// movie asset
		private function makeMovieAssetMat (clipName:String):MovieAssetMaterial
		{
			// linkage id, transparent, animated
			var tempMat:MovieAssetMaterial = new MovieAssetMaterial(clipName,false,true);
			return tempMat;
		}
		
		// bitmap file
		private function makeBitmapFileMat (bitmapUrl:String):BitmapFileMaterial
		{
			var tempMat:BitmapFileMaterial = new BitmapFileMaterial();
			tempMat.smooth = true;
			//bitmapFileMaterial.addEventListener (FileLoadEvent.LOAD_COMPLETE, handleBitmapLoaded,false, 0, true);
			//bitmapFileMaterial.addEventListener (FileLoadEvent.LOAD_ERROR, handleFileLoadError,false, 0, true);
			tempMat.texture = bitmapUrl;

			return tempMat;
		}
		
		// bitmap asset
		private function makeBitmapAssetMat (clipName):BitmapAssetMaterial
		{
			var tempMat:BitmapAssetMaterial = new BitmapAssetMaterial(clipName);
			tempMat.smooth = true;
			tempMat.doubleSided = false;

			return tempMat;
		}
		
		// bitmap material
		private function makeBitmapMat (clipName):BitmapMaterial
		{
			// this part is just creating Bitmap Data to be used as a material.
			var clipBitmap:BitmapData=new BitmapData(clipName.width,clipName.height,true,0x00000000);
			clipBitmap.draw (clipName);

			// create the actual material
			var tempMat:BitmapMaterial = new BitmapMaterial(clipBitmap);
			tempMat.smooth = true;
			tempMat.doubleSided = false;

			return tempMat;
		}
		//----------------------------------------------------------------------------------------------------
		// this converts the position to something relative to the stage x, y (top left) rather than papervisions x,y (center of stage)
		private function convertPosition (clipName:Object,destPos:Object,clipSize:Object):void
		{
			clipName.x=destPos.x + viewport.viewportWidth / 2 - clipSize.width;
			clipName.y=destPos.y + viewport.viewportHeight / 2 - clipSize.height;
		}
		
		//This is the main loop where the camera renders
		private function onEnter (evt:Event):void
		{
			//lets add some movement to our plane so we can see the 3D
			myObject.rotationX-= 2;
			myObject.rotationY+= 3;

			//the render engine requires 3 things, they are easy because they are the other 3 things required in every papervision scene
			//calling this will update the viewport from the cameras point of view.
			render.renderScene (scene,camera,viewport);
		}
	}
}